package com.fr.plugin.http.handler;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.net.http.HttpKit;
import com.fr.base.TemplateUtils;
import com.fr.data.NetworkHelper;
import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.NEWLANConfig;
import com.fr.stable.StringUtils;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.UUID;

public class ALLLan_login1Handler extends BaseHttpHandler {
    @Override
    public RequestMethod getMethod() {
        return null;
    }

    @Override
    public String getPath() {
        return "/lan_login";
    }

    @Override
    public boolean isPublic() {
        return true;
    }

    @Override
    public void handle(HttpServletRequest req, HttpServletResponse res) throws Exception {
        String code = req.getParameter("code");
        NEWLANConfig config = NEWLANConfig.getInstance();
        if (StringUtils.isBlank(code)) {
            String state= UUID.randomUUID().toString();
            String originalURL = WebUtils.getOriginalURL(req);
            String url =String.format("%s/oauth2/authorize?appid=%s&response_type=code&scope=basic_userinfor&state=%s&redirect_uri=%s",config.getAuthUrl(),config.getClientId(),state,originalURL);
            sendRedirect(res, url);
            return;
        }
        String appToken = getAppToken();
        String userToken = getUserToken(appToken, code, WebUtils.getOriginalURL(req));
        String userInfo = getUserInfo(appToken, userToken);
        UserService userService = UserService.getInstance();
        User user = userService.getUserByUserName(userInfo);
        if (user == null) {
            WebUtils.printAsString(res,"user:"+userInfo+" not found" );
            return;
        }
        login(req, res, userInfo);
        String defaultUrl = getDefaultUrl(req);
        if(isMobile(req)){
            LogKit.info("login to mobile {}",defaultUrl+"/url/mobile");
            sendRedirect(res, defaultUrl+"/url/mobile");
            return;
        }
        sendRedirect(res, defaultUrl);
    }

    public static String getDefaultUrl(HttpServletRequest req) {
        StringBuilder url = new StringBuilder();
        try {
            url.append(req.getScheme());
            url.append("://");
            url.append(req.getServerName());
            if (req.getServerPort() != 80) {
                url.append(":");
                url.append(req.getServerPort());
            }
            url.append(TemplateUtils.render("${fineServletURL}"));
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return url.toString();
    }
    public static boolean isMobile(HttpServletRequest req) {
        String[] mobileArray = {"iPhone", "iPad", "android", "windows phone", "xiaomi"};
        String userAgent = req.getHeader("user-agent");
        if (userAgent != null && userAgent.toUpperCase().contains("MOBILE")) {
            for (String mobile : mobileArray) {
                if (userAgent.toUpperCase().contains(mobile.toUpperCase())) {
                    return true;
                }
            }
        }
        return NetworkHelper.getDevice(req).isMobile();
    }

    private void login(HttpServletRequest req, HttpServletResponse res, String username) {
        String token = null;
        try {
            token = LoginService.getInstance().login(req, res, username);
            req.setAttribute("fine_auth_token", token);
        } catch (Exception e) {
        }
    }
    private String getUserInfo(String appToken,String userToken ) throws IOException {
        NEWLANConfig config = NEWLANConfig.getInstance();
        String url=String.format("%s/v1/users/fetch?app_token=%s&user_token=%s",config.getApiUrl(),appToken,userToken);
        String resp = HttpKit.get(url);
        LogKit.info("url:{} getUserInfo resp:{}",url,resp);
        JSONObject entries = new JSONObject(resp);
        return entries.getJSONObject("data").getString("externalId");
    }
    private String getAppToken( ) throws IOException {
        NEWLANConfig config = NEWLANConfig.getInstance();
        String url=String.format("%s/v1/apptoken/create?grant_type=client_credential&appid=%s&secret=%s",config.getApiUrl(),config.getClientId(),config.getClientSecret());
        String resp = HttpKit.get(url);
        LogKit.info("url:{} getAppToken resp:{} ",url,resp);
        JSONObject entries = new JSONObject(resp);
        return entries.getJSONObject("data").getString("app_token");
    }
    private String getUserToken(String appToken,String code ,String re) throws IOException {
        NEWLANConfig config = NEWLANConfig.getInstance();
        String url=String.format("%s/v1/usertoken/create?app_token=%s&grant_type=authorization_code&code=%s&redirect_uri=%s",config.getApiUrl(),appToken,code,re);
        String resp = HttpKit.get(url);
        LogKit.info("url:{} getUserToken resp:{}",url,resp);
        JSONObject entries = new JSONObject(resp);
        return entries.getJSONObject("data").getString("user_token");
    }
    private void sendRedirect(HttpServletResponse res, String url) {
        res.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
        res.setHeader("Location", url);
    }
}
